using System;
using System.Windows.Forms;
using System.Security.Cryptography;
using System.Net.Sockets;
using System.Text.RegularExpressions;


namespace MailTest
{
	public class MailChecker
	{	
		int mailCount;
		public bool DEBUG = false;
		const int APOP_PORT = 110;
		const int IMAP_PORT = 143;
		const int BUFF_SIZE = 400;

		public MailChecker()
		{
			mailCount = 0;
		}


		public int ConnectImapServer(string user, string pass, string hostname)
		{
			TcpClient client = new TcpClient();		
			NetworkStream stream;
			int result = 0;
			
			try
			{
				client.Connect(hostname, IMAP_PORT);
				stream = client.GetStream();
				ReadStream(stream);

				WriteStream(stream, "MAILTEST AUTHENTICATE CRAM-MD5\r\n");
				string inputStr = ReadStream(stream);
				Match m = Regex.Match(inputStr, @"^\+\s(.+)$");
				//Match m;
				if(!(m.Success))
				{
					MessageBox.Show("Error: " + inputStr);	
					result = -1;
					return result;
				}

				string challengeStr = DecodeBase64Str(m.Groups[1].Value);
				string keyStr = CreateHmacMD5(pass, challengeStr);

				WriteStream(stream, EncodeBase64Str(user + " " + keyStr) + "\r\n");			
				if(ReadStream(stream) == "")
				{
					MessageBox.Show("Error: Authentication Error. Maybe username or password is mistaken.");
					result = -1;
					return result;
				}

				WriteStream(stream, "MAILTEST EXAMINE INBOX\r\n");
				string examineStr = ReadStream(stream);		
					
				Match m2 = Regex.Match(examineStr, @"\* (\d+) RECENT");

				if(!(m2.Success))
				{
					MessageBox.Show("Error: " + examineStr);	
					result = -1;
					return result;
				}
				result = int.Parse(m2.Groups[1].Value);
				//if(DEBUG) MessageBox.Show(result.ToString());

				WriteStream(stream, "MAILTEST LOGOUT\r\n");
				ReadStream(stream);
			}
			catch(Exception e)
			{
				result = -1;
				MessageBox.Show(e.ToString());
			}
			return result;
		}


		public int ConncectApopServer(string user, string pass, string hostname)
		{
			TcpClient client = new TcpClient();		
			NetworkStream stream;
			int result = 0;

			try
			{
				client.Connect(hostname, APOP_PORT);
				stream = client.GetStream();
				
				string inputStr = ReadStream(stream);

				Match m = Regex.Match(inputStr, @"^.+(<.+>).*$");
				
				if(!(m.Success))
				{
					MessageBox.Show("Error: " + inputStr);
					result = -1;
					return result;
				}
	
				//if(DEBUG) MessageBox.Show(m.Groups[1].Value);
					
				string passStr = CreateMD5AsStr(m.Groups[1].Value + pass);		
				WriteStream(stream, "APOP " + user + " " + passStr + "\r\n");

				string authStr = ReadStream(stream);

				if(authStr.StartsWith("+OK"))
				{
					//if(DEBUG) MessageBox.Show("Success: " + str);
					Match m2 = Regex.Match(authStr, @"^\+OK \w+ has (\d+) visible messages .+$");
					if(!(m2.Success))
					{
						MessageBox.Show("Error: Can't parse received Data.");
						result = -1;
						return result;
					}
					
					//if(DEBUG) MessageBox.Show("Success: " + m2.Groups[1].Value);
					result = int.Parse(m2.Groups[1].Value);
					if(result >= mailCount)
					{
						int tmpResult = result;
						result = result - mailCount;
						mailCount = tmpResult;
					}
					else
					{
						mailCount = result;
					}
					//if(DEBUG) MessageBox.Show(String.Format("New mail counts are: {0} .", result));

				}
				else
				{
					MessageBox.Show("Error: " + authStr); 
					result = -1;
				}
			
				WriteStream(stream,"QUIT\r\n");
				ReadStream(stream);

				client.Close();
			}
			catch(Exception e)
			{
				MessageBox.Show(e.ToString());
				result = -1;
			}

			return result;
		}
		

		private string RegexToLower(Match m)
		{
			return m.Value.ToLower();
		}


		private string DecodeBase64Str(string str)
		{
			string resultStr;
			byte [] bytes = new byte[BUFF_SIZE];
			bytes = Convert.FromBase64String(str);
			resultStr = Util.ConvertBytesToStr(bytes, bytes.Length);
			
			if(DEBUG) MessageBox.Show(resultStr);
			return resultStr;
		}


		private string EncodeBase64Str(string str)
		{
			byte [] bytes = Util.ConvertStrToBytes(str);
			string resultStr  = Convert.ToBase64String(bytes,0,bytes.Length);
			
			return resultStr;
		}


		private string ReadStream(NetworkStream stream) 
		{
			int cnt=0;
			string str = "";
			byte [] bytes = new byte [BUFF_SIZE];

			try
			{
				if(stream.CanRead)
				{
					cnt = stream.Read(bytes, 0,BUFF_SIZE);
					str = Util.ConvertBytesToStr(bytes,cnt);

					if(DEBUG)MessageBox.Show(str);

					return str;
				}
				else
				{
					MessageBox.Show("Error: Can't Read NetworkStream ");
					return "";
				}
			}
			catch(Exception e)
			{	
				MessageBox.Show(e.ToString());
				return "";
			}
		}


		private bool WriteStream(NetworkStream stream, string str)
		{
			byte [] bytes;
			try
			{
				if(stream.CanWrite)
				{
					bytes = Util.ConvertStrToBytes(str);
					stream.Write(bytes,0,bytes.Length);
					//if(DEBUG)MessageBox.Show("Send: " + str);
					return true;
				}
				else
				{
					MessageBox.Show("Error: Can't Write NetworkStream ");
					return false;
				}

			}
			catch(Exception e)
			{
				if(DEBUG) MessageBox.Show(e.ToString());
				return false;
			}
		}


		private string CreateHmacMD5(string pass, string challenge)
		{
			char [] passChars = new char[64];
			char [] passIpad = new char[64];
			char [] passOpad = new char[64];

			for(int i=0;i<pass.Length;i++)
			{	
				passChars[i] = pass[i];
			}

			for(int i=0;i<64;i++)
			{
				passIpad[i] = passChars[i];
				passOpad[i] = passChars[i];
				passIpad[i] ^= (char)0x36;
				passOpad[i] ^= (char)0x5c;
			}

			byte [] ipadBytes = CreateMD5AsBytes((new String(passIpad)) + challenge);

			string opadStr = CreateMD5AsStr(
				(new String(passOpad)) + Util.ConvertBytesToStr(ipadBytes,ipadBytes.Length));

			return opadStr;

		}


		public byte[] CreateMD5AsBytes (string str)
		{
			byte[] data = Util.ConvertStrToBytes(str);
			MD5 md5 = new MD5CryptoServiceProvider();
			byte[] result = md5.ComputeHash(data);

			return result;
		}


		public string CreateMD5AsStr (string str)
		{
			byte [] bytes = CreateMD5AsBytes(str);
			string result = BitConverter.ToString(bytes);
			
			result = Regex.Replace(result, "-", "");
			result = Regex.Replace(result, @"\w+", new MatchEvaluator(RegexToLower));
				//16i啶\LĂ܂̂ŁCɕύXD
			return result;
		}


	}
}
